@@ -113,6 +113,7 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_ALLOC);
     if ((idp = (__pmLogInDom *)malloc(sizeof(__pmLogInDom))) == NULL)
 	return -oserror();
     idp->next = idp->prior = NULL;
+    idp->indom = lidp->indom;
     idp->stamp = lidp->stamp;		/* struct assignment */
     idp->isdelta = (type == TYPE_INDOM_DELTA);
     idp->buf = indom_buf;
@@ -452,8 +453,8 @@ discard_dup_labelsets(__pmLogLabelSet *idp, const __pmLogLabelSet *idp_next)
  * has been read. At this point we know that the label sets are stored in reverse
  * chronological order.
  */
-static void
-check_dup_labels(const __pmArchCtl *acp)
+void
+__pmCheckDupLabels(const __pmArchCtl *acp)
 {
     __pmLogCtl		*lcp;
     __pmLogLabelSet	*idp, *idp_prior, *idp_next;
@@ -622,6 +623,12 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":2", PM_FAULT_ALLOC);
     return __pmHashAdd((int)dp->pmid, (void *)dp, &lcp->hashpmid);
 }
 
+/*
+ * return 0 for added OK
+ * return 1 for duplicate name and PMID
+ * return 2 for duplicate name but different PMID ... see note below
+ * return <0 for error
+ */
 int
 __pmLogAddPMNSNode(__pmArchCtl *acp, pmID pmid, const char *name)
 {
@@ -638,7 +645,7 @@ __pmLogAddPMNSNode(__pmArchCtl *acp, pmID pmid, const char *name)
      */
     sts = __pmAddPMNSNode(lcp->pmns, pmid, name);
     if (sts == PM_ERR_PMID)
-	sts = 0;
+	sts = 1;
     return sts;
 }
 
@@ -682,7 +689,24 @@ __pmLogLoadMeta(__pmArchCtl *acp)
     int			i;
     int			len;
     char		name[MAXPATHLEN];
-    
+    int			nrec[TYPE_MAX+1] = { 0 };
+    char		*recname[TYPE_MAX+1] = { "bad", "desc", "indomv2", "labelv2", "text", "indom", "delta", "label" };
+
+    if (pmDebugOptions.logmeta)
+	fprintf(stderr, "__pmLogLoadMeta(acp=%p => name=%s)",
+	    acp, acp->ac_log->name);
+
+    if (acp->ac_meta_loaded == 1) {
+	/*
+	 * only load metadata once per archive
+	 */
+	if (pmDebugOptions.logmeta)
+	    fprintf(stderr, " already loaded, skipped\n");
+	return 0;
+    }
+    if (pmDebugOptions.logmeta)
+	fputc('\n', stderr);
+
     if (lcp->pmns == NULL) {
 	if ((sts = __pmNewPMNS(&(lcp->pmns))) < 0)
 	    goto end;
@@ -714,12 +738,18 @@ __pmLogLoadMeta(__pmArchCtl *acp)
 		sts = PM_ERR_LOGREC;
 	    goto end;
 	}
-	if (pmDebugOptions.logmeta) {
+	if (pmDebugOptions.logmeta && pmDebugOptions.desperate) {
 	    char    strbuf[15];
 	    fprintf(stderr, "__pmLogLoadMeta: record len=%d, type=%s (%d) @ offset=%d\n",
 		h.len, __pmLogMetaTypeStr_r(h.type, strbuf, sizeof(strbuf)),
 		h.type, (int)(__pmFtell(f) - sizeof(__pmLogHdr)));
 	}
+	if (pmDebugOptions.logmeta) {
+	    if (h.type < TYPE_DESC || h.type > TYPE_MAX)
+		nrec[0]++;
+	    else
+		nrec[h.type]++;
+	}
 	rlen = h.len - (int)sizeof(__pmLogHdr) - (int)sizeof(int);
 	if (h.type == TYPE_DESC) {
 	    pmDesc		desc;
@@ -803,15 +833,25 @@ __pmLogLoadMeta(__pmArchCtl *acp)
 		    goto end;
 		}
 		name[len] = '\0';
-		if (pmDebugOptions.logmeta) {
-		    char	strbuf[20];
-		    fprintf(stderr, "%s: PMID: %s name: %s\n",
-			    "__pmLogLoadMeta",
-			    pmIDStr_r(desc.pmid, strbuf, sizeof(strbuf)), name);
-		}
 
 		/* Add the new PMNS node into this context */
-		if ((sts = __pmLogAddPMNSNode(acp, desc.pmid, name)) < 0)
+		sts = __pmLogAddPMNSNode(acp, desc.pmid, name);
+		if (pmDebugOptions.logmeta) {
+		    char	strbuf[20];
+		    fprintf(stderr, "%s: PMID: %s name: %s",
+			    "__pmLogLoadMeta",
+			    pmIDStr_r(desc.pmid, strbuf, sizeof(strbuf)), name);
+		    if (sts == 1)
+			fprintf(stderr, " (duplicate)");
+		    else if (sts == 2)
+			fprintf(stderr, " (PMID mismatch)");
+		    else if (sts < 0)
+			fprintf(stderr, " (error=%d)", sts);
+		    fputc('\n', stderr);
+		}
+		if (sts == 0)
+		    lcp->numpmid++;
+		if (sts < 0)
 		    goto end;
 	    }/*for*/
 	}
@@ -948,7 +988,8 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":16", PM_FAULT_ALLOC);
 				"__pmLogLoadMeta", h.type, (int)(__pmFtell(f) - sizeof(check)));
 
 	    }
-	    return PM_ERR_RECTYPE;
+	    sts = PM_ERR_RECTYPE;
+	    goto done;
 	}
 	n = (int)__pmFread(&check, 1, sizeof(check), f);
 	check = ntohl(check);
@@ -969,11 +1010,6 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":16", PM_FAULT_ALLOC);
     }/*for*/
 end:
 
-    /* Check for duplicate label sets. */
-    check_dup_labels(acp);
-    
-    __pmFseek(f, (long)__pmLogLabelSize(lcp), SEEK_SET);
-
     if (sts == 0) {
 	if (numpmid == 0) {
 	    if (pmDebugOptions.logmeta) {
@@ -981,9 +1017,21 @@ end:
 	    }
 	    sts = PM_ERR_LOGREC;
 	}
-	else
-	    __pmFixPMNSHashTab(lcp->pmns, numpmid, 1);
     }
+
+done:
+    if (pmDebugOptions.logmeta) {
+	int	tot = 0;
+	fprintf(stderr, "__pmLogLoadMeta => %d, records", sts);
+	for (i = 0; i <= TYPE_MAX; i++) {
+	    if (nrec[i] > 0) {
+		fprintf(stderr, " %s:%d", recname[i], nrec[i]);
+		tot += nrec[i];
+	    }
+	}
+	fprintf(stderr, " total:%d\n", tot);
+    }
+
     return sts;
 }
 
@@ -1135,6 +1183,7 @@ __pmLogUndeltaInDom(pmInDom indom, __pmLogInDom *idp)
 	int	i;		/* index over last full indom */
 	int	j;		/* index over delta indom */
 	int	k;		/* index over new full indom we're building */
+	assert (didp->isdelta == 1);
 	numinst = didp->next->numinst;
 	for (j = 0; j < didp->numinst; j++) {
 	    if (didp->namelist[j] != NULL)
@@ -1169,6 +1218,7 @@ __pmLogUndeltaInDom(pmInDom indom, __pmLogInDom *idp)
 		    /* add new instance in correct sorted position */
 		    if (pmDebugOptions.logmeta && pmDebugOptions.desperate)
 			fprintf(stderr, "[%d] add from [%d] inst %d \"%s\"\n", k, j, didp->instlist[j], didp->namelist[j]);
+		    assert (k < numinst);
 		    instlist[k] = didp->instlist[j];
 		    namelist[k] = didp->namelist[j];
 		    k++;
@@ -1180,6 +1230,7 @@ __pmLogUndeltaInDom(pmInDom indom, __pmLogInDom *idp)
 		/* copy instance from end of old indom */
 		if (pmDebugOptions.logmeta && pmDebugOptions.desperate)
 		    fprintf(stderr, "[%d] dup from [%d] inst %d \"%s\"\n", k, i, tidp->instlist[i], tidp->namelist[i]);
+		assert(k < numinst);
 		instlist[k] = tidp->instlist[i];
 		namelist[k] = tidp->namelist[i];
 		k++;
@@ -1189,6 +1240,7 @@ __pmLogUndeltaInDom(pmInDom indom, __pmLogInDom *idp)
 		/* add new instance at end of indom */
 		if (pmDebugOptions.logmeta && pmDebugOptions.desperate)
 		    fprintf(stderr, "[%d] append from [%d] inst %d \"%s\"\n", k, j, didp->instlist[j], didp->namelist[j]);
+		assert(k < numinst);
 		instlist[k] = didp->instlist[j];
 		namelist[k] = didp->namelist[j];
 		k++;
@@ -1670,10 +1722,9 @@ pmGetInDomArchive_ctx(__pmContext *ctxp, pmInDom indom, int **instlist, char ***
 	    /* Need to "un-delta" this delta indom record */
 	    __pmLogUndeltaInDom(indom, idp);
 	}
-	if (idp->numinst > HASH_THRESHOLD) {
+	if (idp->numinst > HASH_THRESHOLD && big_indom == 0) {
 	    big_indom = 1;
 	    reset_ihash();
-	    break;
 	}
     }
 
@@ -1736,7 +1787,23 @@ PM_FAULT_POINT("libpcp/" __FILE__ ":9", PM_FAULT_ALLOC);
 int
 pmGetInDomArchive(pmInDom indom, int **instlist, char ***namelist)
 {
-    return pmGetInDomArchive_ctx(NULL, indom, instlist, namelist);
+    int		sts;
+
+    if (pmDebugOptions.pmapi) {
+	char    dbgbuf[20];
+	fprintf(stderr, "pmGetInDomArchive(%s,...) <:", pmInDomStr_r(indom, dbgbuf, sizeof(dbgbuf)));
+    }
+    sts = pmGetInDomArchive_ctx(NULL, indom, instlist, namelist);
+    if (pmDebugOptions.pmapi) {
+	fprintf(stderr, ":> returns ");
+	if (sts >= 0)
+	    fprintf(stderr, "%d\n", sts);
+	else {
+	    char	errmsg[PM_MAXERRMSGLEN];
+	    fprintf(stderr, "%s\n", pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+	}
+    }
+    return sts;
 }
 
 void
