@@ -161,6 +161,13 @@ extern int	_creds_timeout;
 /* flag for context label changes */
 extern int	labelChanged;
 
+/* client limits */
+PMCD_DATA extern unsigned	maxmetrics;	/* max PMIDs per pmFetch */
+PMCD_DATA extern unsigned	maxctx;		/* max number of contexts per client */
+
+/* input PDU size (bytes) */
+PMCD_DATA extern unsigned	maxinpdusize;
+
 /* global PMCD PMDA variables */
 
 /*
