@@ -48,6 +48,10 @@ fi
 # error messages should go to stderr, not the GUI notifiers
 unset PCP_STDERR
 
+# want mode for mkdir below to reliably be rwxrwxr-x (775)
+#
+umask 022
+
 # default message log file
 PROGLOG=$PCP_LOG_DIR/pmlogger/$prog.log
 USE_SYSLOG=true
@@ -318,18 +322,15 @@ fi
 
 if [ "$PROGLOG" = "/dev/tty" ]
 then
-    # special case for debugging ... no salt away previous, no chown, no exec
+    # special case for debugging ... no salt away previous
     #
     :
 else
     # After argument checking, everything must be logged to ensure no mail is
     # accidentally sent from cron.  Close stdout and stderr, then open stdout
-    # as our logfile and redirect stderr there too.  Create the log file with
-    # correct ownership first.
+    # as our logfile and redirect stderr there too.
     #
     [ -f "$PROGLOG" ] && mv "$PROGLOG" "$PROGLOG.prev"
-    touch "$PROGLOG"
-    chown $PCP_USER:$PCP_GROUP "$PROGLOG" >/dev/null 2>&1
     exec 1>"$PROGLOG" 2>&1
 fi
 
@@ -357,10 +358,14 @@ $VERBOSE && echo ARCHIVEPATH=$ARCHIVEPATH
 [ -z "$REPORTDIR" ] && REPORTDIR="$PCP_SA_DIR"
 $VERBOSE && echo REPORTDIR=$REPORTDIR
 
-# Create output directory - if this fails due to permissions we exit later
+# Create output directory - if this fails for any reason we exit later
 #
+# mode rwxrwxr-x is the default for pcp:pcp dirs
+umask 002
 [ -d "$REPORTDIR" ] \
-    || { mkdir -p "$REPORTDIR" 2>/dev/null; chmod 0775 "$REPORTDIR" 2>/dev/null; }
+    || mkdir -p -m 0775 "$REPORTDIR" 2>/dev/null
+# reset the default mode to rw-rw-r- for files
+umask 022
 
 # Default output file is the day of month for yesterday in REPORTDIR
 #
@@ -419,15 +424,15 @@ _report()
     else
 	if grep 'PM_ERR_NAME' $tmp/err >/dev/null 2>&1
 	then
-	    metric=`$PCP_AWK_PROG <$tmp/err '{ print $3 }'`
+	    metric=`$PCP_AWK_PROG <$tmp/err '/PM_ERR_NAME/ { print $3; exit }'`
 	    echo "-- no report for config \"$_conf\" because the metric \"$metric\" is not in the archive" >>$REPORTFILE
 	elif grep 'PM_ERR_INDOM_LOG' $tmp/err >/dev/null 2>&1
 	then
-	    metric=`$PCP_AWK_PROG <$tmp/err '{ print $3 }'`
+	    metric=`$PCP_AWK_PROG <$tmp/err '/PM_ERR_INDOM_LOG/ { print $3; exit }'`
 	    echo "-- no report for config \"$_conf\" because there are no values for any instance of the metric \"$metric\" in the archive" >>$REPORTFILE
 	elif grep 'PM_ERR_BADDERIVE' $tmp/err >/dev/null 2>&1
 	then
-	    metric=`$PCP_AWK_PROG <$tmp/err '{ print $3 }'`
+	    metric=`$PCP_AWK_PROG <$tmp/err '/PM_ERR_BADDERIVE/ { print $3; exit }'`
 	    echo "-- no report for config \"$_conf\" because one or more metrics for the derived metric \"$metric\" is not in the archive" >>$REPORTFILE
 	else
 	    cat $tmp/err >>$REPORTFILE
