@@ -266,7 +266,8 @@ validate_metrics(void)
     int			error;
     int			sts;
     time_t		now;
-    char		buf1[20], buf2[20];
+			/* buffers long enough for __pmUnitsStr_r() */
+    char		buf1[60], buf2[60];
 
     time(&now);
     fprintf(stderr, "%s: Validating metrics after PMCD state changed at %s",
@@ -370,9 +371,13 @@ validate_metrics(void)
 		new_desc.units.scaleTime != old_desc->units.scaleTime ||
 		new_desc.units.scaleCount != old_desc->units.scaleCount) {
 		++error;
-		fprintf(stderr, "Units of metric \"%s\" has changed from %s to %s\n",
+		fprintf(stderr, "Units of metric \"%s\" has changed from [%d,%d,%d,%d,%d,%d] %s to [%d,%d,%d,%d,%d,%d] %s\n",
 			tp->t_namelist[index],
+			old_desc->units.dimSpace, old_desc->units.dimTime, old_desc->units.dimCount,
+			old_desc->units.scaleSpace, old_desc->units.scaleTime, old_desc->units.scaleCount,
 			pmUnitsStr_r(&old_desc->units, buf1, sizeof(buf1)),
+			new_desc.units.dimSpace, new_desc.units.dimTime, new_desc.units.dimCount,
+			new_desc.units.scaleSpace, new_desc.units.scaleTime, new_desc.units.scaleCount,
 			pmUnitsStr_r(&new_desc.units, buf2, sizeof(buf2)));
 	    }
 	} /* loop over metrics */
